@@ -36,7 +36,9 @@
 #include <QPixmap>
 #include <QDrag>
 #include <QWindow>
+#if QT_CONFIG(opengl)
 #include <QOpenGLWindow>
+#endif
 
 #include <QtTest/QtTest>
 #include <QtWaylandClient/private/qwaylandintegration_p.h>
@@ -107,6 +109,7 @@ public:
     QPoint mousePressPos;
 };
 
+#if QT_CONFIG(opengl)
 class TestGlWindow : public QOpenGLWindow
 {
     Q_OBJECT
@@ -136,6 +139,7 @@ void TestGlWindow::paintGL()
     glClear(GL_COLOR_BUFFER_BIT);
     ++paintGLCalled;
 }
+#endif // QT_CONFIG(opengl)
 
 class tst_WaylandClient : public QObject
 {
@@ -145,7 +149,7 @@ public:
         : compositor(c)
     {
         QSocketNotifier *notifier = new QSocketNotifier(compositor->waylandFileDescriptor(), QSocketNotifier::Read, this);
-        connect(notifier, SIGNAL(activated(int)), this, SLOT(processWaylandEvents()));
+        connect(notifier, SIGNAL(activated(QSocketDescriptor)), this, SLOT(processWaylandEvents()));
         // connect to the event dispatcher to make sure to flush out the outgoing message queue
         connect(QCoreApplication::eventDispatcher(), &QAbstractEventDispatcher::awake, this, &tst_WaylandClient::processWaylandEvents);
         connect(QCoreApplication::eventDispatcher(), &QAbstractEventDispatcher::aboutToBlock, this, &tst_WaylandClient::processWaylandEvents);
@@ -167,11 +171,6 @@ public slots:
     }
 
 private slots:
-    void primaryScreen();
-    void screens();
-    void addScreenWithGeometryChange();
-    void windowScreens();
-    void removePrimaryScreen();
     void createDestroyWindow();
     void activeWindowFollowsKeyboardFocus();
     void events();
@@ -181,7 +180,9 @@ private slots:
     void dontCrashOnMultipleCommits();
     void hiddenTransientParent();
     void hiddenPopupParent();
+#if QT_CONFIG(opengl)
     void glWindow();
+#endif // QT_CONFIG(opengl)
     void longWindowTitle();
     void longWindowTitleWithUtf16Characters();
 
@@ -189,117 +190,6 @@ private:
     MockCompositor *compositor = nullptr;
 };
 
-void tst_WaylandClient::primaryScreen()
-{
-    compositor->setOutputMode(screenSize);
-    QTRY_COMPARE(QGuiApplication::primaryScreen()->size(), screenSize);
-}
-
-void tst_WaylandClient::screens()
-{
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-    compositor->sendAddOutput();
-    QTRY_COMPARE(QGuiApplication::screens().size(), 2);
-    QSharedPointer<MockOutput> secondOutput;
-    QTRY_VERIFY(secondOutput = compositor->output(1));
-    compositor->sendRemoveOutput(secondOutput);
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-}
-
-//QTBUG-62044
-void tst_WaylandClient::addScreenWithGeometryChange()
-{
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-    const QRect oldGeometry = QGuiApplication::primaryScreen()->geometry();
-    compositor->sendAddOutput();
-
-    // Move the primary screen to the right
-    const QRect newGeometry(QPoint(screenSize.width(), 0), screenSize);
-    Q_ASSERT(oldGeometry != newGeometry);
-    compositor->sendOutputGeometry(compositor->output(0), newGeometry);
-
-    QTRY_COMPARE(QGuiApplication::screens().size(), 2);
-    QTRY_COMPARE(QGuiApplication::primaryScreen()->geometry(), newGeometry);
-
-    compositor->sendRemoveOutput(compositor->output(1));
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-
-    // Move the screen back
-    compositor->sendOutputGeometry(compositor->output(0), oldGeometry);
-    QTRY_COMPARE(QGuiApplication::primaryScreen()->geometry(), oldGeometry);
-}
-
-void tst_WaylandClient::windowScreens()
-{
-    QSharedPointer<MockOutput> firstOutput;
-    QTRY_VERIFY(firstOutput = compositor->output());
-
-    TestWindow window;
-    window.show();
-
-    QSharedPointer<MockSurface> surface;
-    QTRY_VERIFY(surface = compositor->surface());
-    compositor->sendShellSurfaceConfigure(surface);
-
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-    QScreen *primaryScreen = QGuiApplication::screens().first();
-    QCOMPARE(window.screen(), primaryScreen);
-
-    compositor->sendAddOutput();
-
-    QTRY_COMPARE(QGuiApplication::screens().size(), 2);
-    QScreen *secondaryScreen = QGuiApplication::screens().at(1);
-    QVERIFY(secondaryScreen);
-
-    window.setScreen(secondaryScreen);
-    QCOMPARE(window.screen(), secondaryScreen);
-
-    QSharedPointer<MockOutput> secondOutput;
-    QTRY_VERIFY(secondOutput = compositor->output(1));
-    compositor->sendSurfaceEnter(surface, firstOutput);
-
-    compositor->sendSurfaceEnter(surface, secondOutput);
-    QTRY_COMPARE(window.screen(), primaryScreen);
-
-    compositor->sendSurfaceLeave(surface, firstOutput);
-    QTRY_COMPARE(window.screen(), secondaryScreen);
-
-    compositor->sendRemoveOutput(secondOutput);
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-    QCOMPARE(window.screen(), primaryScreen);
-}
-
-void tst_WaylandClient::removePrimaryScreen()
-{
-    QSharedPointer<MockOutput> firstOutput;
-    QTRY_VERIFY(firstOutput = compositor->output());
-
-    TestWindow window;
-    window.show();
-
-    QSharedPointer<MockSurface> surface;
-    QTRY_VERIFY(surface = compositor->surface());
-    compositor->sendShellSurfaceConfigure(surface);
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-    QScreen *primaryScreen = QGuiApplication::screens().first();
-    QCOMPARE(window.screen(), primaryScreen);
-
-    compositor->sendAddOutput();
-
-    QTRY_COMPARE(QGuiApplication::screens().size(), 2);
-    QTRY_COMPARE(QGuiApplication::primaryScreen()->virtualSiblings().size(), 2);
-    QScreen *secondaryScreen = QGuiApplication::screens().at(1);
-    QVERIFY(secondaryScreen);
-
-    compositor->sendRemoveOutput(firstOutput);
-    QTRY_COMPARE(QGuiApplication::screens().size(), 1);
-
-    compositor->sendMousePress(surface, window.frameOffset() + QPoint(10, 10));
-    QTRY_COMPARE(window.mousePressEventCount, 1);
-    compositor->sendMouseRelease(surface);
-    QTRY_COMPARE(window.mouseReleaseEventCount, 1);
-}
-
 void tst_WaylandClient::createDestroyWindow()
 {
     TestWindow window;
@@ -575,6 +465,7 @@ void tst_WaylandClient::hiddenPopupParent()
     QTRY_VERIFY(compositor->surface());
 }
 
+#if QT_CONFIG(opengl)
 void tst_WaylandClient::glWindow()
 {
     QSKIP("Skipping GL tests, as not supported by all CI systems: See https://bugreports.qt.io/browse/QTBUG-65802");
@@ -600,6 +491,7 @@ void tst_WaylandClient::glWindow()
     testWindow->setVisible(false);
     QTRY_VERIFY(!compositor->surface());
 }
+#endif // QT_CONFIG(opengl)
 
 void tst_WaylandClient::longWindowTitle()
 {
@@ -623,7 +515,8 @@ void tst_WaylandClient::longWindowTitleWithUtf16Characters()
 
 int main(int argc, char **argv)
 {
-    setenv("XDG_RUNTIME_DIR", ".", 1);
+    QTemporaryDir tmpRuntimeDir;
+    setenv("XDG_RUNTIME_DIR", tmpRuntimeDir.path().toLocal8Bit(), 1);
     setenv("QT_QPA_PLATFORM", "wayland", 1); // force QGuiApplication to use wayland plugin
 
     MockCompositor compositor;
